"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogLevel = void 0;
const MethodFactory_1 = require("./MethodFactory");
const PrefixFactory_1 = require("./PrefixFactory");
const defaults = {
    factory: void 0,
    level: 'warn',
    name: (+new Date()).toString(),
    prefix: void 0
};
class LogLevel {
    constructor(options) {
        this.type = 'LogLevel';
        this.options = Object.assign({}, defaults, options);
        this.methodFactory = options.factory;
        if (!this.methodFactory) {
            const factory = options.prefix
                ? new PrefixFactory_1.PrefixFactory(this, options.prefix)
                : new MethodFactory_1.MethodFactory(this);
            this.methodFactory = factory;
        }
        if (!this.methodFactory.logger) {
            this.methodFactory.logger = this;
        }
        this.name = options.name || '<unknown>';
        this.level = this.options.level ?? 'trace';
    }
    get factory() {
        return this.methodFactory;
    }
    set factory(factory) {
        factory.logger = this;
        this.methodFactory = factory;
        this.methodFactory.replaceMethods(this.level);
    }
    disable() {
        this.level = this.levels.SILENT;
    }
    enable() {
        this.level = this.levels.TRACE;
    }
    get level() {
        return this.currentLevel;
    }
    set level(logLevel) {
        const level = this.methodFactory.distillLevel(logLevel);
        if (level === false || level == null) {
            throw new RangeError(`loglevelnext: setLevel() called with invalid level: ${logLevel}`);
        }
        this.currentLevel = level;
        this.methodFactory.replaceMethods(level);
        if (typeof console === 'undefined') {
            process.stdout.write('loglevelnext: console is undefined. The log will produce no output.\n');
        }
        else if (level < this.levels.SILENT) {
            console.warn(`The log level has been set to a value less than 'silent'. The log will produce no output.`);
        }
    }
    get levels() {
        return this.methodFactory.levels;
    }
}
exports.LogLevel = LogLevel;
//# sourceMappingURL=LogLevel.js.map