"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const LogLevel_1 = require("./LogLevel");
const MethodFactory_1 = require("./MethodFactory");
const PrefixFactory_1 = require("./PrefixFactory");
const factories = Symbol('log-factories');
class DefaultLogger extends LogLevel_1.LogLevel {
    constructor() {
        super({ name: 'default' });
        this.cache = { default: this };
        this[factories] = { MethodFactory: MethodFactory_1.MethodFactory, PrefixFactory: PrefixFactory_1.PrefixFactory };
    }
    get factories() {
        return this[factories];
    }
    get loggers() {
        return this.cache;
    }
    create(opts) {
        let options;
        if (typeof opts === 'string') {
            options = { name: opts };
        }
        else {
            options = Object.assign({}, opts);
        }
        if (!options.id) {
            options.id = options.name.toString();
        }
        const { name, id } = options;
        const defaults = { level: this.level };
        if (typeof name !== 'string' || !name || !name.length) {
            throw new TypeError('You must supply a name when creating a logger.');
        }
        let logger = this.cache[id];
        if (!logger) {
            logger = new LogLevel_1.LogLevel(Object.assign({}, defaults, options));
            this.cache[id] = logger;
        }
        return logger;
    }
}
exports.default = new DefaultLogger();
//# sourceMappingURL=index.js.map